@@ -1,20 +1,3 @@
-# -*- coding:utf-8; python-indent:2; indent-tabs-mode:nil -*-
-
-# Copyright 2013 Google Inc. All Rights Reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
-
 """Functions for optimizing pytd syntax trees.
 
    pytd files come from various sources, and are typically redundant (duplicate
@@ -29,7 +12,6 @@ import logging
 from pytype import utils
 from pytype.pytd import abc_hierarchy
 from pytype.pytd import booleq
-from pytype.pytd import mro
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import type_match
@@ -43,7 +25,7 @@ class RenameUnknowns(visitors.Visitor):
   """Give unknowns that map to the same set of concrete types the same name."""
 
   def __init__(self, mapping):
-    super(RenameUnknowns, self).__init__()
+    super().__init__()
     self.name_to_cls = {name: hash(cls) for name, cls in mapping.items()}
     self.cls_to_canonical_name = {
         cls: name for name, cls in self.name_to_cls.items()}
@@ -87,7 +69,7 @@ class RemoveRedundantSignatures(visitors.Visitor):
   """
 
   def __init__(self, hierarchy):
-    super(RemoveRedundantSignatures, self).__init__()
+    super().__init__()
     self.match = type_match.TypeMatch(hierarchy.GetSuperClasses(),
                                       any_also_is_bottom=False)
     self.subst = {}
@@ -141,7 +123,7 @@ class SimplifyUnions(visitors.Visitor):
     return pytd_utils.JoinTypes(union.type_list)
 
 
-class _ReturnsAndExceptions(object):
+class _ReturnsAndExceptions:
   """Mutable class for collecting return types and exceptions of functions.
 
   The collecting is stable: Items are kept in the order in which they were
@@ -489,7 +471,7 @@ class ApplyOptionalArguments(visitors.Visitor):
     return f.Replace(signatures=tuple(new_signatures))
 
 
-class SuperClassHierarchy(object):
+class SuperClassHierarchy:
   """Utility class for optimizations working with superclasses."""
 
   def __init__(self, superclasses):
@@ -569,13 +551,13 @@ class SimplifyUnionsWithSuperclasses(visitors.Visitor):
   """
 
   def __init__(self, hierarchy):
-    super(SimplifyUnionsWithSuperclasses, self).__init__()
+    super().__init__()
     self.hierarchy = hierarchy
 
   def VisitUnionType(self, union):
     c = collections.Counter()
     for t in set(union.type_list):
-      # TODO(rechen): How can we make this work with GenericType?
+      # TODO(b/159052794): How can we make this work with GenericType?
       if isinstance(t, pytd.GENERIC_BASE_TYPE):
         c += collections.Counter(self.hierarchy.ExpandSubClasses(str(t)))
     # Below, c[str[t]] can be zero - that's the default for non-existent items
@@ -595,7 +577,7 @@ class FindCommonSuperClasses(visitors.Visitor):
   """
 
   def __init__(self, hierarchy):
-    super(FindCommonSuperClasses, self).__init__()
+    super().__init__()
     self.hierarchy = hierarchy
 
   def VisitUnionType(self, union):
@@ -645,8 +627,8 @@ class CollapseLongUnions(visitors.Visitor):
   """
 
   def __init__(self, max_length=7):
-    assert isinstance(max_length, six.integer_types)
-    super(CollapseLongUnions, self).__init__()
+    assert isinstance(max_length, int)
+    super().__init__()
     self.generic_type = pytd.AnythingType()
     self.max_length = max_length
 
@@ -663,7 +645,7 @@ class AdjustGenericType(visitors.Visitor):
   """Changes the generic type from "object" to "Any"."""
 
   def __init__(self):
-    super(AdjustGenericType, self).__init__()
+    super().__init__()
     self.old_generic_type = pytd.ClassType("__builtin__.object")
     self.new_generic_type = pytd.AnythingType()
 
@@ -712,14 +694,14 @@ class AddInheritedMethods(visitors.Visitor):
     if any(base for base in cls.parents if isinstance(base, pytd.NamedType)):
       raise AssertionError("AddInheritedMethods needs a resolved AST")
     # Filter out only the types we can reason about.
-    # TODO(kramm): Do we want handle UnionTypes and GenericTypes at some point?
+    # TODO(b/159052794): Do we want handle UnionTypes and GenericTypes?
     bases = [base.cls
              for base in cls.parents
              if isinstance(base, pytd.ClassType)]
     # Don't pull in methods that are named the same as existing methods in
     # this class, local methods override parent class methods.
     names = {m.name for m in cls.methods} | {c.name for c in cls.constants}
-    # TODO(kramm): This should do full-blown MRO.
+    # TODO(b/159052405): This should do full-blown MRO.
     adjust_self = visitors.AdjustSelf(force=True)
     adjust_self.class_types.append(visitors.ClassAsType(cls))
     new_methods = list(cls.methods)
@@ -736,104 +718,6 @@ class AddInheritedMethods(visitors.Visitor):
                        constants=tuple(new_constants))
 
 
-class RemoveInheritedMethods(visitors.Visitor):
-  """Removes methods from classes if they also exist in their superclass.
-
-  E.g. this changes
-        class A:
-            def f(self, y: int) -> bool
-        class B(A):
-            def f(self, y: int) -> bool
-  to
-        class A:
-            def f(self, y: int) -> bool
-        class B(A):
-            pass
-  .
-  """
-
-  def __init__(self):
-    super(RemoveInheritedMethods, self).__init__()
-    self.class_to_stripped_signatures = {}
-
-  def _StrippedSignatures(self, t):
-    """Given a class, list method name + signature without "self".
-
-    Args:
-      t: A pytd.Type.
-
-    Returns:
-      A set of name + signature tuples, with the self parameter of the
-      signature removed.
-    """
-    if not isinstance(t, pytd.ClassType):
-      # For union types, generic types etc., inheritance is more complicated.
-      # Be conservative and default to not removing methods inherited from
-      # those.
-      return {}
-
-    stripped_signatures = {}
-    for method in t.cls.methods:
-      for sig in method.signatures:
-        if (sig.params and
-            sig.params[0].name == "self" and
-            isinstance(sig.params[0].type, pytd.ClassType)):
-          stripped_signatures[method.name] = (
-              sig.Replace(params=sig.params[1:]), method.is_abstract)
-    return stripped_signatures
-
-  def _FindNameAndSig(self, classes, name, sig):
-    """Find a tuple(name, signature) in all methods of a type/class."""
-    if classes:
-      t = classes[0]
-      classes = classes[1:]
-      if t not in self.class_to_stripped_signatures:
-        self.class_to_stripped_signatures[t] = self._StrippedSignatures(t)
-      if name in self.class_to_stripped_signatures[t]:
-        return sig == self.class_to_stripped_signatures[t][name]
-      return self._FindNameAndSig(classes, name, sig)
-    return False
-
-  def _MaybeRemoveSignature(self, name, sig, is_abstract):
-    """Visit a Signature and return None if we can remove it."""
-    if (not sig.params or
-        sig.params[0].name != "self" or
-        not isinstance(sig.params[0].type, pytd.ClassType)):
-      return sig  # Not a method
-    cls = sig.params[0].type.cls
-    if cls is None:
-      # TODO(kramm): Remove once pytype stops generating ClassType(name, None).
-      return sig
-    try:
-      if self._FindNameAndSig(
-          mro.GetBasesInMRO(cls), name,
-          (sig.Replace(params=sig.params[1:]), is_abstract)):
-        return None  # remove (see VisitFunction)
-    except mro.MROError:
-      return sig
-    return sig
-
-  def _MaybeDeleteFunction(self, f):
-    """Visit a Function and return None if we can remove it."""
-    signatures = tuple(self._MaybeRemoveSignature(f.name, sig, f.is_abstract)
-                       for sig in f.signatures)
-    if any(signatures):
-      if signatures.count(None):
-        return f.Replace(
-            signatures=tuple(s for s in signatures if s is not None))
-      else:
-        return f  # unchanged
-    else:
-      return None  # delete function
-
-  def VisitClass(self, cls):
-    methods = tuple(self._MaybeDeleteFunction(m) for m in cls.methods)
-    if methods.count(None):
-      return cls.Replace(methods=tuple(m for m in methods if m is not None))
-    else:
-      return cls  # unchanged
-
-
 class PullInMethodClasses(visitors.Visitor):
   """Simplifies classes with only a __call__ function to just a method.
 
@@ -849,7 +733,7 @@ class PullInMethodClasses(visitors.Visitor):
   """
 
   def __init__(self):
-    super(PullInMethodClasses, self).__init__()
+    super().__init__()
     self._module = None
     self._total_count = collections.defaultdict(int)
     self._processed_count = collections.defaultdict(int)
@@ -996,7 +880,7 @@ class TypeParameterScope(visitors.Visitor):
   """Common superclass for optimizations that track type parameters."""
 
   def __init__(self):
-    super(TypeParameterScope, self).__init__()
+    super().__init__()
     self.type_params_stack = [{}]
 
   def EnterClass(self, cls):
@@ -1049,7 +933,7 @@ class MergeTypeParameters(TypeParameterScope):
   """
 
   def __init__(self):
-    super(MergeTypeParameters, self).__init__()
+    super().__init__()
     self.type_param_union = None
 
   def _AppendNew(self, l1, l2):
@@ -1061,13 +945,13 @@ class MergeTypeParameters(TypeParameterScope):
 
   def EnterSignature(self, node):
     # Necessary because TypeParameterScope also defines this function
-    super(MergeTypeParameters, self).EnterSignature(node)
+    super().EnterSignature(node)
     assert self.type_param_union is None
     self.type_param_union = collections.defaultdict(list)
 
   def LeaveSignature(self, node):
     # Necessary because TypeParameterScope also defines this function
-    super(MergeTypeParameters, self).LeaveSignature(node)
+    super().LeaveSignature(node)
     self.type_param_union = None
 
   def VisitUnionType(self, u):
@@ -1110,7 +994,6 @@ class MergeTypeParameters(TypeParameterScope):
     else:
       # It's a function type parameter that appears in a union with other
       # function type parameters.
-      # TODO(kramm): We could merge those, too.
       return [item]
 
   def VisitSignature(self, sig):
@@ -1181,6 +1064,5 @@ def Optimize(node,
   node = node.Visit(SimplifyContainers())
   if builtins and can_do_lookup:
     node = visitors.LookupClasses(node, builtins, ignore_late_types=True)
-    node = node.Visit(RemoveInheritedMethods())
     node = node.Visit(RemoveRedundantSignatures(hierarchy))
   return node
